<?php
class TagParser {
    private $identifier;
    private $openclose = true;
    public $tags = false;
    public function __construct($identifier,$openclose=true) {
        $this->identifier = $identifier;
        $this->openclose = $openclose;
    }
    public function parse($content) {
        // simple search first to capture full tags:
        $regex ='/{'.$this->identifier.'(?<params>.*?)}'; // open tag and params
        if($this->openclose) {
            $regex.='(?<content>.*?)'; // content if it exists
            $regex.='(?<closetag>{\/'.$this->identifier.'})'; // close tag if it exists
        }
        $regex.='/'; // open tag and params
        $matches = array();
        preg_match_all($regex,$content,$matches);
        if(count($matches)) $this->tags = array();
        foreach($matches[0] as $key=>$match) {
            $this->tags[$key]=new TagParserItem($match);
            if(
                    array_key_exists('params',$matches) && 
                    array_key_exists($key,$matches['params'])
            ) {
                $params = new DOMDocument;
                $params->loadHTML('<br '.trim($matches['params'][$key].' />'));
                $paramsxml = simplexml_import_dom($params->documentElement);
                if(count($paramsxml->body->br->attributes())) {
                    $this->tags[$key]->params = array();
                    foreach($paramsxml->body->br->attributes() as $attribute=>$value) {
                        $this->tags[$key]->params[$attribute]=(strlen($value->__toString())?$value->__toString():true);
                    }
                }
            }
            if(
                    array_key_exists('closetag',$matches) && 
                    array_key_exists($key,$matches['closetag'])
            ) {
                $this->tags[$key]->content = $matches['content'][$key];
            }
        }
    }
    public function replace($key,$content,$replacement='') {
        if(!is_numeric($key)) return;
        if(!$this->tags[$key]->getContent(false)) return;
        $regex='/'.preg_quote($this->tags[$key]->tag,'/').'/';
        return preg_replace($regex,$replacement,$content,1);
    }
}
class TagParserItem {
    public $tag;
    public $params = false;
    public $content = false;
    public function __construct($tag) {
        $this->tag = $tag;
    }
    public function getParam($name,$default) {
        return isset($this->params[$name])?$this->params[$name]:$default;
    }
    public function getContent($default='') {
        return $this->content?$this->content:$default;
    }
}

?>